.MODEL  Small
.CODE

;Name    :Keysnse2.com
;Usage   :Passes Keypresses and shift state to batch files.
;        :keysnse2 with no parameters defaults to Keypress which reads any key
;        :pressed including extended keys. Must be called again for second part
;        :of errorlevel number remaining in keybuffer if extended keypressed.
;        :The errorlevel value is equal to the ASCII value of the key pressed.
;        :Consult your manuals for ASCII values of keypresses.
;
;        :If An Invalid First Parameter Is Passed, keysnse2 Defaults To (A).
;
;        :Parameters are: Alpabetical for Keyinput, no parameter same as A.
;        :Ie: keysnse2 (same as keysnse2 a) ; keysnse2 A ; keysnse2 c.
;        :Parameters may be typed in upper or lower case.
;
;        :A-anykey, B-(A) with buffer flushed, C-anykey caps conversion,
;        :D-(C) buffer flushed, E-alpha-num only with caps conversion,
;        :F-(E) with buffer flushed G-(E) but also reads function keys.
;        :H-(G) with buffer flushed. The G & H Parameters directly return
;        :errorlevels for the unshifted and shifted function key presses.
;        :keysnse2 does not have to be called a second time. The errorlevel
;        :returned is shown below.
;
;           F-Key         Errlvl         Shifted Errlvl
;
;        : F1              001                011
;        : F2              002                012
;        : F3              003                013
;        : F4              004                014
;        : F5              005                015
;        : F6              006                016
;        : F7              007                017
;        : F8              008                018
;        : F9              009                019
;        : F10             010                020
;
;        : F11             021                023
;        : F12             022                024
;
;        :The non flushed and flushed options are available for each type of 
;        :keypress level for convienince or safety. The non flushed options
;        :allow keypresses to be passed through the key buffer, without wait.
;        :If you know ahead what keys to push, they can be all entered ahead
;        :of time and accepted by keysnse2 at the appropriate time. If you 
;        :press a wrong key however, it could be disasterous in some cases.
;        :The flushed state therfore is provided so that you can require
;        :a keypress to be accepted only after prompted to do so. This can
;        :be used at critical points such as options that would erase files,
;        :format a disk etc., with a warning message presented and waiting for
;        :a (Y) or (N) pressed. If an option such as above was accidentaly
;        :keyed in followed by a (Y), keysnse2 b, d, f, h would flush the (Y),
;        :and any other keys, from the buffer and wait for a keypress. For the
;        :a, c, e, g options, the (Y) would be retrievd from the buffer and
;        :acted upon without wait. Both keysnse2 e and keysnse2 g will flush
;        :the buffer and wait for a new keypress if an invalid keypress is 
;        :passed from the key buffer.
;
;        :Parameters are: Numerical for Shiftstate
;        :Ie: keysnse2 1 ; keysnse2 3 ; keysnse2 0;2 ; keysnse2 341
;
;        :Parameters may be grouped together, ie: keysnse2 1 3 5 2, to test for
;        :combinations. The above example would return an errorlevel of 1 if the
;        :Ins, Num, Alt, & Cap keys are all depressed. It would return a level
;        :of 0 if any of those keys were not depressed. It disreguards any other
;        :keys that might be depressed. If you wanted to test that those four
;        :only are pressed, follow the test of those four with a test of the
;        :remaining four useing the 0 parameter. 0 folowed by key parameters
;        :will test for keys not depressed. You could follow that test with
;        :keysnse2 (individual parameters), to see what if any keys are pressed.
;        :Parameters may be separated by {;}, {,}, or spaces. They also may be
;        :chained, ie; keysnse2 3521 , keysnse2 52673 etc.
;
;        :9-Returns cumulative errorlevel, ie: 129 if ins key and r-shft pressed,
;        :098 for caps,num,l-shft; 255 for all.
;        :Parameter Number and Errorlevel values for each key is:
;
;        :Para         Key           Errlvl
;        :
;        : 1           Ins             128
;        : 2           Caps            64
;        : 3           Num             32
;        : 4           Scrl            16
;        : 5           Alt             8
;        : 6           Ctrl            4
;        : 7           L-Shft          2
;        : 8           R-Shft          1
;        : 9           cumul           total
;        : 0        Not pressed folowed by keys to test.
;
;        :Batchfile example for regular and extended keypress'.
;
;        :entrky
;        keysnse2 B
;        if errorlevel 0 if not errorlevel 1 goto highkey
;        if errorlevel 65 if not errorlevel 66 goto Apress
;        goto keyend
;        :highkey
;        keysnse2
;        if errorlevel 65 if not errorlevel 66 goto F7press
;        goto keyend
;        :Apress
;        echo The A key was pressed.
;        goto keyend
;        :F7press
;        echo The F7 key was pressed.
;        :keyend
;
;        :Separate Blocks for extended keypress errorlevel testing so that the
;        :resulting action can be differentiated, ie: F7 and A both return 065
;        :as final errorlevel.
;
;        :Batchfile example for shiftstate
;
;         echo off
;         cls
;         echo Test.bat
;         echo Testing for keysnse2 option
;         keysnse2 4
;         if errorlevel 1 if not errorlevel 2 goto speed
;         echo 
;         echo Scroll Key not depressed. Waiting for Response.
;         echo Press A key B key or Esc to exit.
;         :top
;         keysnse2 f
;         if errorlevel 27 if not errorlevel 28 goto esc
;         if errorlevel 65 if not errorlevel 66 goto aky
;         if errorlevel 66 if not errorlevel 67 goto bky
;         echo No good. Not a key tested for.
;         goto top
;         :speed
;         echo 
;         echo Speed key.......Scroll is depressed. Skipped Options.
;         goto end
;         :aky
;         echo The A key Was Pressed.
;         goto end
;         :bky
;         echo The B key Was Pressed.
;         goto end
;         :esc
;         echo Esc. Pressed.
;         :end
;
;

keysnse2        PROC

    mov     si,80h
    cld
    lodsb
    cmp     al,0Dh
    je      parm_E1
    call    Seek_nxt_parm

    cmp     al,30h
    je      not_deprs
    cmp     al,31h
    jb      parm_E1
    cmp     al,39h
    jna     Dpr_kys_dwn
    call    Cap_conv

Dpr_kys_dwn:
    cmp     al,39h
    ja      Ent_parms
    cmp     al,39h
    jne     Chk_for_dpr
    call    Any_dpr_key
    jmp     All_done

Chk_for_dpr:
    jmp     do_chk_dpr

Ent_parms:
    cmp     al,42h
    jz      parm_E2
    cmp     al,43h
    jz      parm_E3
    cmp     al,44h
    jz      parm_E4
    cmp     al,45h
    jz      parm_E5
    cmp     al,46h
    jz      parm_E6
    cmp     al,47h
    jz      parm_E7
    cmp     al,48h
    jz      parm_E8

parm_E1:
    call    No_flush
    jmp     All_done

parm_E2:
    call    Buf_flush
    jmp     All_done

parm_E3:
    call    No_flush
    call    Cap_conv
    jmp     All_done

parm_E4:
    call    Buf_flush
    call    Cap_conv
    jmp     All_done

parm_E5:
    call    No_flush
    cmp     al,1Bh
    je      All_done
    cmp     al,00h
    jnz     E5_Fl_jump
    call    No_flush
    jmp     parm_E5
E5_Fl_jump:
    call    Valid_keys
    cmp     al,0ffh
    je      parm_E6
    jmp     All_done

not_deprs:
    mov     cx,3000h
    jmp     do_again

parm_E6:
    call    Buf_flush
    cmp     al,1Bh
    je      All_done
    jmp     E5_Fl_jump

parm_E7:
    call    No_flush
    jmp     Parm_E7_rtrn
parm_E8:
    call    Buf_flush
Parm_E7_rtrn:
    cmp     al,1Bh
    je      All_done
    cmp     al,00h
    jne     Tst_for_valid
    call    Extd_keys
    jmp     Ext_kytst_done
Tst_for_valid:
    call    Valid_keys
Ext_kytst_done:
    cmp     al,0ffh
    je      parm_E8
    jmp     All_done

close_out:
    mov     ax,bx
All_done:
    mov     ah,4ch
    int     21h

do_again:
    call    Seek_nxt_parm
do_chk_dpr:
    cmp     al,0dh
    jz      close_out
    mov     dl,al
    cmp     dl,31h
    jb      do_again
    cmp     dl,38h
    jna     yes_no
    jmp     do_again

yes_no:
    sub     dl,30h
    mov     cl,dl
    mov     dx,0000h
    call    Any_dpr_key
    mov     bl,al
    rcl     bl,cl
    adc     dl,00h
    cmp     ch,30h
    je      non_prsd
    cmp     dl,01h
    jmp     chk_prsd

non_prsd:
    cmp     dl,00h

chk_prsd:
    jz      its_good
    mov     ax,4c00h
    jmp     all_done

its_good:
    mov     bx,4c01h
    mov     ax,0000h
    jmp     do_again

keysnse2    ENDP

Extd_keys      PROC
    mov     ah,08
    int     21h
    cmp     al,3bh
    jb      Ext_bad
    cmp     al,44h
    jna     Ext_sub58
    cmp     al,54h
    jb      Ext_extra
    cmp     al,5dh
    jna     Ext_sub73
    jmp     Ext_extra
Ext_sub58:
    sub     al,3ah
    jmp     Ext_good
Ext_sub73:
    sub     al,49h
    jmp     Ext_good
Ext_extra:
    cmp     al,85h
    jb      Ext_bad
    cmp     al,88h
    ja      Ext_bad
    sub     al,70h
    jmp     Ext_good
Ext_bad:
    mov     al,0ffh
Ext_good:
    ret
Extd_keys      ENDP

Valid_keys      PROC
    cmp     al,30h
    jb      Get_new_key
    cmp     al,5ah
    jna     No_cap_conv
    call    Cap_conv
No_cap_conv:
    cmp     al,3ah
    jb      Conv_proc_done
    cmp     al,41h
    jb      Get_new_key
    jmp     Conv_proc_done
Get_new_key:
    mov     al,0ffh
Conv_proc_done:
    ret
Valid_keys      ENDP

No_flush       PROC
    mov     ah,08h
    int     21h
    ret
No_flush       ENDP

Buf_flush       PROC
    mov     al,08h
    mov     ah,0ch
    int     21h
    ret
Buf_flush       ENDP

Seek_nxt_parm   PROC
nxt_skip:
    lodsb
    cmp     al,2ch
    je      nxt_skip
    cmp     al,3bh
    je      nxt_skip
    cmp     al,20h
    je      nxt_skip
    ret
Seek_nxt_parm   ENDP

Any_Dpr_key       PROC
    mov     ah,02h
    int     16h
    ret
Any_Dpr_key       ENDP

Cap_conv        PROC
    cmp     al,61h
    jb      Conv_done
    cmp     al,7ah
    ja      Conv_done
    sub     al,20h
Conv_done:
     ret
Cap_conv        ENDP

        END keysnse2
